require 'active_support/core_ext/string/inflections'

module Endicia
  class LabelResponse
    attr_accessor :image,
                  :status,
                  :tracking_number,
                  :final_postage,
                  :transaction_date_time,
                  :transaction_id,
                  :postmark_date,
                  :postage_balance,
                  :postage_price,
                  :pic,
                  :error_message,
                  :reference_id,
                  :reference_id2,
                  :reference_id3,
                  :reference_id4,
                  :requester_id,
                  :cost_center,
                  :request_body,
                  :request_url,
                  :response_body

    def initialize(result)
      self.response_body = filter_response_body(result.body.dup)
      data = result["LabelRequestResponse"] || {}
      data.each do |k, v|
        if k == 'Base64LabelImage'
          k = "image"
        elsif k == 'Label'
          # when customs form is included in xml, Endicia returns the image in a different format
          k = "image"
          # There can be one or many <Image> elements in <Label>. MultiXml parses multiples into an array.
          v = [v["Image"]].flatten.first["__content__"]
        end
        setter = :"#{k.tableize.singularize}="
        send(setter, v) if !k['xmlns'] && respond_to?(setter)
      end
    end

    # True if this result represents a successful label response
    def label_generated?
      status && status.to_i == 0
    end

    # True if this is an error and it's a "critical" one, meaning the ability
    # to generate further labels could be impeded by the information received back
    def critical_error?
      CRITICAL_ERROR_CODES.include?(status.to_i)
    end

  private

    CRITICAL_ERROR_CODES = [
      1101,
      2011,
      2012,
      2013,
      2014,
      2015,
      2020,
      2021,
      12101,
      12103,
      12104,
      12115,
      12116,
      12117,
      12120,
      12130,
      12133,
      12503,
      12505,
      12507,
      12515,
      12525,
      100201,
    ].freeze

    def filter_response_body(string)
      # Strip image data for readability:
      string.gsub(/<((Base64Label)?Image( PartNumber=\"\d+\")?)>.+?<\/((Base64Label)?Image)>/, '<\1>[data]</\4>')
    end
  end
end
