require 'ostruct'
require 'nokogiri'

module Endicia
  class RefundResponse

    attr_accessor :success,         # boolean: true if all tracking numbers are approved or false otherwise
                                    # (Probably better to use tracking_numbers to check each individually)
                  :form_number,     # Form Number for refunded label
                  :raw_response,    # The XML response as a string
                  :parsed_response, # A Nokogiri document of the XML response
                  :tracking_numbers # An array with information about all requested tracking numbers
                                    # Each entry is an OpenStruct with:
                                    #   pic_number: '123456789',         # the tracking number you requested
                                    #   approved:   true,                # or false
                                    #   message:    "the message"        # message describing success or failure

    def initialize(response)
      @success = true
      @tracking_numbers = []
      @raw_response = response.body

      @parsed_response = Nokogiri::XML(response.body)
      refund_response = @parsed_response.xpath('/RefundResponse')
      @form_number = refund_response.at('FormNumber').content

      refund_list = refund_response.at('RefundList')

      refund_list.xpath('//PICNumber').each do |refund_node|
        approved = refund_node.at('IsApproved').content.try(:strip).try(:upcase) == "YES"
        @tracking_numbers << OpenStruct.new(pic_number: refund_node.children.first.text.try(:strip),
                                            approved:   approved,
                                            message:    refund_node.at('ErrorMsg').content.try(:strip))
        @success = @success && approved
      end
    end

  end
end
